// Copyright 2012 Google Inc. All Rights Reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * Implement a subtype of this class and pass it as the argument to
 * world.setContactFilter to provide collision filtering.
 * In other words, you can implement this class if you want finer control over
 * contact creation.
 */

part of box2d;

class ContactFilter {
  ContactFilter();

  /**
   * Return true if contact calculations should be performed between these two
   * shapes.
   */
  bool shouldCollide(Fixture fixtureA, Fixture fixtureB) {
    final Filter filterA = fixtureA.filter;
    final Filter filterB = fixtureB.filter;

    if (filterA.groupIndex != 0 && filterA.groupIndex == filterB.groupIndex)
      return filterA.groupIndex > 0;

    return (filterA.maskBits & filterB.categoryBits) != 0 &&
           (filterA.categoryBits & filterB.maskBits) != 0;
  }
}

